@@ -37,7 +37,7 @@
 #define GCONF_KEY_LISTS GCONF_KEY_PREFIX"/lists"
 #define GCONF_KEY_KEYS GCONF_KEY_PREFIX"/keys/"
 
-#define HSPLIT 320
+#define HSPLIT 380
 #define VSPLIT 260
 
 #define CONTROL(app) ((KMPlayer::Control *)app->control)
@@ -110,7 +110,7 @@ void cb_row_activated (GtkTreeView *treeview, GtkTreePath *path, GtkTreeViewColu
 void cb_playlist_edit_commit (GtkButton * button, Application * app);
 void cb_openUrls (void *app, int argc, gchar **argv);
 void cb_view_area_realized (GtkWidget *widget, Application * app);
-void cb_view_area_expose (GtkWidget *widget, GdkEventExpose *event, Application * app);
+gboolean cb_view_area_expose (GtkWidget *widget, GdkEventExpose *event, Application * app);
 void cb_view_area_configure (GtkWidget *widget, GdkEventConfigure *event, Application * app);
 void cb_view_area_button_press (GtkWidget *widget, GdkEventButton *event, Application * app);
 void cb_playlistMenuActivate (GtkWidget *w, GdkEvent *e, Application *app);
