@@ -31,6 +31,7 @@ struct TransTypeInfo;
 namespace KMPlayer {
 
 class ImageActor;
+class Expression;
 
 typedef std::vector <String> StringList;
 
@@ -84,7 +85,7 @@ public:
 class KMPLAYER_NO_EXPORT Runtime {
 public:
     enum TimingState {
-        TimingsInit = 0, TimingsInitialized,
+        TimingsInit = 0, TimingsInitialized, TimingsDisabled,
         timings_began, timings_started, TimingsTransIn, timings_paused,
         timings_stopped, timings_freezed
     };
@@ -102,6 +103,7 @@ public:
         DurStart = (int) MsgEventStarted,
         DurEnd = (int) MsgEventStopped,
         DurMedia = (int)MsgMediaFinished,
+        DurStateChanged = (int)MsgStateChanged,
         DurTransition,
         DurLastDuration
     };
@@ -148,6 +150,7 @@ public:
     TimingState timingstate;
     TimingState unpaused_state;
     int repeat_count;
+    String expr;
     ConnectionList m_StartListeners;        // Element about to be started
     ConnectionList m_StartedListeners;      // Element is started
     ConnectionList m_StoppedListeners;      // Element stopped
@@ -216,6 +219,7 @@ namespace SMIL {
 
 const short id_node_smil = 100;
 const short id_node_head = 101;
+const short id_node_state = 102;
 const short id_node_layout = 103;
 const short id_node_root_layout = 104;
 const short id_node_region = 105;
@@ -232,6 +236,9 @@ const short id_node_ref = 123;
 const short id_node_brush = 124;
 const short id_node_smil_text = 125;
 const short id_node_text_styling = 126;
+const short id_node_set_value = 127;
+const short id_node_new_value = 128;
+const short id_node_del_value = 129;
 const short id_node_set = 132;
 const short id_node_animate = 133;
 const short id_node_animate_color = 134;
@@ -245,6 +252,7 @@ const short id_node_span = 145;
 const short id_node_p = 146;
 const short id_node_anchor = 150;
 const short id_node_area = 151;
+const short id_node_state_data = 151;
 const short id_node_first = id_node_smil;
 const short id_node_first_timed_mrl = id_node_body;
 const short id_node_last_timed_mrl = id_node_animate_motion;
@@ -274,6 +282,7 @@ public:
     static Smil * findSmilNode (Node * node);
 
     NodePtrW layout_node;
+    NodePtrW state_node;
 };
 
 /**
@@ -290,6 +299,34 @@ public:
     bool expose () const;
 };
 
+/**
+ * Defines state, should reside below 'head' element
+ */
+class KMPLAYER_NO_EXPORT State : public Element {
+public:
+    enum Where { before, after, child };
+
+    State (NodePtr & d);
+
+    virtual Node *childFromTag (const String & tag);
+    virtual void closed ();
+    virtual void activate ();
+    virtual void parseParam (const TrieString & name, const String & value);
+    virtual void deactivate ();
+    virtual void message (MessageType msg, void *content=NULL);
+    virtual void *role (RoleType msg, void *content=NULL);
+    KDE_NO_EXPORT virtual const char * nodeName () const { return "state"; }
+    virtual bool expose () const { return false; }
+
+    void newValue (Node *ref, Where w, const String &name, const String &val);
+    void setValue (Node *ref, const String &value);
+    void delValue (Node *ref);
+
+    ConnectionList m_StateChangeListeners;        // setValue changed a value
+    PostponePtr postpone_lock;                    // pause while loading src
+    MediaInfo *media_info;
+};
+
 /**
  * Defines region layout, should reside below 'head' element
  */
@@ -559,6 +596,7 @@ public:
     KDE_NO_EXPORT const char * nodeName () const { return "switch"; }
     // Condition
     void begin ();
+    void init ();
     void deactivate ();
     void reset ();
     void message (MessageType msg, void *content=NULL);
@@ -661,13 +699,13 @@ public:
     bool trans_out_active;
 
 protected:
+    virtual void prefetch ();
     virtual void clipStart ();
     virtual void clipStop ();
 
     MouseListeners mouse_listeners;
     ConnectionList m_MediaAttached;
     ConnectionList m_TransformedIn;        // transIn ready
-    ConnectionLink region_paint;           // attached region needs painting
     ConnectionLink region_attach;          // attached to region
     ConnectionLink document_postponed;     // pause audio/video accordantly
     ConnectionLink transition_updater;
@@ -682,7 +720,7 @@ public:
     virtual void finish ();
     virtual void accept (Visitor *);
     virtual bool expose () const;
-    virtual void parseParam (const TrieString &, const String &);
+    virtual void prefetch ();
     virtual void clipStart ();
 };
 
@@ -693,8 +731,8 @@ public:
     PlayType playType () { return play_type_image; }
     virtual void activate ();
     virtual void accept (Visitor *);
-    virtual void parseParam (const TrieString &, const String &);
     virtual void message (MessageType msg, void *content=NULL);
+    virtual void prefetch ();
 };
 
 class KMPLAYER_NO_EXPORT TextMediaType : public MediaType {
@@ -704,6 +742,7 @@ public:
     virtual void init ();
     virtual void accept (Visitor *);
     virtual void parseParam (const TrieString &, const String &);
+    virtual void prefetch ();
 
     String font_name;
     int font_size;
@@ -765,6 +804,58 @@ public:
     String tag;
 };
 
+class KMPLAYER_NO_EXPORT StateValue : public Element {
+public:
+    ~StateValue ();
+
+    virtual void init ();
+    virtual void activate ();
+    virtual void finish ();
+    virtual void deactivate ();
+    virtual void reset ();
+    virtual void parseParam (const TrieString &name, const String &value);
+    virtual void message (MessageType msg, void *content=NULL);
+    virtual void *role (RoleType msg, void *content=NULL);
+    virtual bool expose () const { return false; }
+protected:
+    StateValue (NodePtr &d, short _id);
+
+    String value;
+    NodePtrW state;
+    Expression *ref;
+    Runtime *runtime;
+};
+
+class KMPLAYER_NO_EXPORT NewValue : public StateValue {
+public:
+    NewValue (NodePtr &d) : StateValue (d, id_node_new_value) {}
+
+    virtual void init ();
+    virtual void begin ();
+    virtual void parseParam (const TrieString &name, const String &value);
+    KDE_NO_EXPORT const char *nodeName () const { return "newvalue"; }
+
+private:
+    String name;
+    SMIL::State::Where where;
+};
+
+class KMPLAYER_NO_EXPORT SetValue : public StateValue {
+public:
+    SetValue (NodePtr &d) : StateValue (d, id_node_set_value) {}
+
+    virtual void begin ();
+    KDE_NO_EXPORT const char *nodeName () const { return "setvalue"; }
+};
+
+class KMPLAYER_NO_EXPORT DelValue : public StateValue {
+public:
+    DelValue (NodePtr &d) : StateValue (d, id_node_del_value) {}
+
+    virtual void begin ();
+    KDE_NO_EXPORT const char *nodeName () const { return "delvalue"; }
+};
+
 class KMPLAYER_NO_EXPORT AnimateGroup : public Element {
 public:
     ~AnimateGroup ();
@@ -843,6 +934,7 @@ protected:
 class KMPLAYER_NO_EXPORT Animate : public AnimateBase {
 public:
     Animate (NodePtr &doc);
+
     virtual void init ();
     virtual void begin ();
     virtual void finish ();
