@@ -274,7 +274,7 @@ bool UnixProcess::start (const String &pwd, const String &cmdline) {
         g_io_channel_set_encoding (pout, NULL, NULL);
         flags = g_io_channel_get_flags (pout);
         g_io_channel_set_flags (pout, (GIOFlags)(flags|G_IO_FLAG_NONBLOCK), 0L);
-        g_io_channel_set_buffer_size (pout, 80);
+        g_io_channel_set_buffer_size (pout, 0);
         wout = g_io_add_watch (pout, G_IO_IN, watchProcessStdout, this);
         return true;
     }
@@ -780,9 +780,6 @@ KMPLAYER_NO_EXPORT bool MPlayer::sendCommand (const String & cmd) {
 static NSNotify ns_notify;
 
 extern "C" {
-static void npEmbedded (void *ndata) {
-    ((NpPlayer *) ndata)->pluginEmbedded ();
-}
 
 static void npSetDimension (void *ndata, int w, int h) {
     NpPlayer *player = (NpPlayer *) ndata;
@@ -860,13 +857,10 @@ KDE_NO_CDTOR_EXPORT NpPlayer::NpPlayer (ProcessNotify * ctrl)
    argv (NULL),
    processing(false) {
     plugins = g_tree_new ((GCompareFunc)::strcmp);
-    if (!ns_notify.embedded) {
-        ns_notify.embedded = npEmbedded;
-        ns_notify.setDimension = npSetDimension;
-        ns_notify.getUrl = npGetUrl;
-        ns_notify.evaluate = nsEvaluate;
-        ns_notify.finishStream = npFinish;
-    }
+    ns_notify.setDimension = npSetDimension;
+    ns_notify.getUrl = npGetUrl;
+    ns_notify.evaluate = nsEvaluate;
+    ns_notify.finishStream = npFinish;
 }
 
 KDE_NO_CDTOR_EXPORT NpPlayer::~NpPlayer () {
@@ -943,6 +937,7 @@ KMPLAYER_NO_EXPORT bool NpPlayer::play (NodePtr node) {
     g_tree_traverse (params, copyParams, G_IN_ORDER, tuple);
     g_tree_destroy (params);
 
+    nppInstanceTopUrl (m_url);
     npdata = nppInstanceOpen (mime, argc, argn, argv, (void *)this, &ns_notify);
     if (!npdata) {
         g_tree_destroy (streams);
@@ -951,11 +946,11 @@ KMPLAYER_NO_EXPORT bool NpPlayer::play (NodePtr node) {
         warningLog() << "failed to create new instance" << endl;
         return false;
     }
-    nppInstanceStart ((const char *)m_url, mime, npdata);
     return true;
 }
 
 KMPLAYER_NO_EXPORT void NpPlayer::pluginEmbedded () {
+    nppInstanceStart ((const char *)m_url, mime, npdata);
     setState (Playing);
 }
 
