@@ -37,7 +37,7 @@ class ViewAreaPrivate;
  */
 class KMPLAYER_NO_EXPORT ViewArea {
 public:
-    ViewArea ();
+    ViewArea (Application *app);
     ~ViewArea ();
     void reset ();
     KDE_NO_EXPORT bool isFullScreen () const { return m_fullscreen; }
@@ -63,8 +63,8 @@ public:
     int width () const;
     int height () const;
     GtkWidget *viewWidget () const { return widget; }
-    GdkWindow *videoWindow () const { return video; }
     GdkWindow *videoInputWindow () const { return video_input; }
+    unsigned long videoXId () const;
     Mrl *videoMrl () { return video_node ? video_node->mrl () : NULL; }
     void setOutputWidget (Application *, GtkWidget * w);
     void setControlsOverlap (bool b) { m_controls_overlap = b; }
@@ -79,8 +79,9 @@ private:
     IRect m_av_geometry;
     float aspect;
     IRect m_repaint_rect;
+    Application *m_app;
     GtkWidget *widget;
-    GdkWindow *video;
+    GtkWidget *socket;
     GdkWindow *video_input;
     SurfacePtr surface;
     NodePtrW video_node;
